@@ -50,11 +50,11 @@
  * The format record for the XPM file format:
  */
 
-static int		ChanMatchXPM _ANSI_ARGS_((Tcl_Interp *interp,Tcl_Channel chan,
+static int		ChanMatchXPM _ANSI_ARGS_((Tcl_Channel chan,
 			    Tcl_Obj *fileName, Tcl_Obj *format, int *widthPtr,
-			    int *heightPtr));
-static int      	ObjMatchXPM _ANSI_ARGS_((Tcl_Interp *interp,Tcl_Obj *dataObj,
-		            Tcl_Obj *format, int *widthPtr, int *heightPtr));
+			    int *heightPtr, Tcl_Interp *interp));
+static int      	ObjMatchXPM _ANSI_ARGS_((Tcl_Obj *dataObj,
+		            Tcl_Obj *format, int *widthPtr, int *heightPt, Tcl_Interp *interp));
 static int		ChanReadXPM  _ANSI_ARGS_((Tcl_Interp *interp,
 			    Tcl_Channel chan, Tcl_Obj *fileName,
 			    Tcl_Obj *format, Tk_PhotoHandle imageHandle,
@@ -69,8 +69,7 @@ static int		CommonReadXPM _ANSI_ARGS_((Tcl_Interp *interp, MFile *handle,
 			    int destX, int destY, int width, int height,
 			    int srcX, int srcY));
 static int	        StringWriteXPM _ANSI_ARGS_((Tcl_Interp *interp,
-               		    Tcl_DString *dataPtr, Tcl_Obj *format,
-		            Tk_PhotoImageBlock *blockPtr));
+			    Tcl_Obj *format, Tk_PhotoImageBlock *blockPtr));
 static int              FileWriteXPM _ANSI_ARGS_((Tcl_Interp *interp,
                             char *fileName, Tcl_Obj *format,
                             Tk_PhotoImageBlock *blockPtr));
@@ -158,13 +157,13 @@ Gets(handle, buffer, size)
  *----------------------------------------------------------------------
  */
 static int
-ObjMatchXPM(interp, dataObj, format, widthPtr, heightPtr)
-    Tcl_Interp *interp;
+ObjMatchXPM(dataObj, format, widthPtr, heightPtr, interp)
     Tcl_Obj *dataObj;		/* The data supplied by the image */
     Tcl_Obj *format;		/* User-specified format object, or NULL. */
     int *widthPtr, *heightPtr;	/* The dimensions of the image are
 				 * returned here if the file is a valid
 				 * raw XPM file. */
+    Tcl_Interp *interp;
 {
     int numColors, byteSize;
     MFile handle;
@@ -177,7 +176,7 @@ ObjMatchXPM(interp, dataObj, format, widthPtr, heightPtr)
     return ReadXPMFileHeader(&handle, widthPtr, heightPtr, &numColors, &byteSize);
 }
 
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -197,14 +196,14 @@ ObjMatchXPM(interp, dataObj, format, widthPtr, heightPtr)
  */
 
 static int
-ChanMatchXPM(interp, chan, fileName, format, widthPtr, heightPtr)
-    Tcl_Interp *interp;
+ChanMatchXPM(chan, fileName, format, widthPtr, heightPtr, interp)
     Tcl_Channel chan;		/* The image channel, open for reading. */
     Tcl_Obj *fileName;		/* The name of the image file. */
     Tcl_Obj *format;		/* User-specified format object, or NULL. */
     int *widthPtr, *heightPtr;	/* The dimensions of the image are
 				 * returned here if the file is a valid
 				 * raw XPM file. */
+    Tcl_Interp *interp;
 {
     int numColors, byteSize;
     MFile handle;
@@ -216,7 +215,7 @@ ChanMatchXPM(interp, chan, fileName, format, widthPtr, heightPtr)
 
     return ReadXPMFileHeader(&handle, widthPtr, heightPtr, &numColors, &byteSize);
 }
-
+
 
 /*
  *----------------------------------------------------------------------
@@ -447,7 +446,7 @@ CommonReadXPM(interp, handle, format, imageHandle, destX, destY,
 	}
 	p += byteSize * srcX + 1;
 	pixelPtr = block.pub.pixelPtr;
-	
+
 	for (i = 0; i < width; ) {
 	    unsigned int col;
 
@@ -462,14 +461,14 @@ CommonReadXPM(interp, handle, format, imageHandle, destX, destY,
 	        col = (int)Tcl_GetHashValue(hPtr);
 	    else
 	        col = (int)0;
-	
+
 	    /*
 	     * we've found a non-transparent pixel, let's search the next
 	     * transparent pixel and copy this block to the image
 	     */
 	    if (col) {
 	        int len = 0, j;
-		
+
 		j = i;
 		pixelPtr = block.pub.pixelPtr;
 		do {
@@ -478,7 +477,7 @@ CommonReadXPM(interp, handle, format, imageHandle, destX, destY,
 		    i++;
 		    len++;
 		    p += byteSize;
-		
+
 		    if (i < width) {
 		        memcpy((char *) &color1, p, byteSize);
 			hPtr = Tcl_FindHashEntry(&colorTable, (char *) color1);
@@ -488,7 +487,7 @@ CommonReadXPM(interp, handle, format, imageHandle, destX, destY,
 			    col = (int)0;
 		    }
 		} while ((i < width) && col);
-		Tk_PhotoPutBlock(imageHandle, &block.pub, destX+j, destY, len, 1);
+		Tk_PhotoPutBlock(imageHandle, &block.pub, destX+j, destY, len, 1, TK_PHOTO_COMPOSITE_SET);
 	    } else {
 	        p += byteSize;
 	        i++;
@@ -503,7 +502,7 @@ CommonReadXPM(interp, handle, format, imageHandle, destX, destY,
     return TCL_OK;
 }
 
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -771,7 +770,7 @@ static char * GetColor(colorDefn, colorName, type_ret)
 	if (GetType(colorDefn, &dummy) == NULL) {
 	    /* the next string should also be considered as a part of a color
 	     * name */
-	
+
 	    while (*colorDefn && isspace(UCHAR(*colorDefn))) {
 		*p++ = *colorDefn++;
 	    }
@@ -833,15 +832,15 @@ FileWriteXPM(interp, fileName, format, blockPtr)
  *
  *----------------------------------------------------------------------
  */
-static int	
-StringWriteXPM(interp, dataPtr, format, blockPtr)
+static int
+StringWriteXPM(interp, format, blockPtr)
     Tcl_Interp *interp;
-    Tcl_DString *dataPtr;
     Tcl_Obj *format;
     Tk_PhotoImageBlock *blockPtr;
 {
     int result;
     Tcl_DString data;
+    Tcl_DString *dataPtr;
     ImgFixStringWriteProc(&data, &interp, &dataPtr, &format, &blockPtr);
     result = CommonWriteXPM(interp, "unknown", dataPtr, format, blockPtr);
     if ((result == TCL_OK) && (dataPtr == &data)) {
@@ -1034,7 +1033,7 @@ CommonWriteXPM(interp, fileName, dataPtr, format, blockPtr)
 		/* make transparent pixel */
 		memcpy(buffer, "    ", chars_per_pixel);
 	    }
-	    pp += blockPtr->pixelSize;	
+	    pp += blockPtr->pixelSize;
 	    WRITE(buffer);
 	}
 	if (y == blockPtr->height - 1) {
@@ -1053,3 +1052,4 @@ CommonWriteXPM(interp, fileName, dataPtr, format, blockPtr)
     }
     return TCL_OK;
 }
+
