@@ -38,7 +38,7 @@
  */
 
 static int		AlignParseProc _ANSI_ARGS_((ClientData clientData,
-			    Tcl_Interp *interp, Tk_Window tkwin, 
+			    Tcl_Interp *interp, Tk_Window tkwin,
 			    Tcl_Obj *value, char *widgRec, int offset));
 static Tcl_Obj *	AlignPrintProc _ANSI_ARGS_((ClientData clientData,
 			    Tk_Window tkwin, char *widgRec, int offset,
@@ -374,7 +374,7 @@ EmbImageConfigure(textPtr, eiPtr, argc, argv)
 	    if (new > count) {
 		count = new;
 	    }
-	    if (len == (int) strlen(haveName)) {
+	    if (len == strlen(haveName)) {
 	    	conflict = 1;
 	    }
 	}
@@ -468,7 +468,7 @@ AlignParseProc(clientData, interp, tkwin, valuearg, widgRec, offset)
  */
 
 	/* ARGSUSED */
-Arg
+Tcl_Obj *
 AlignPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
     ClientData clientData;		/* Ignored. */
     Tk_Window tkwin;			/* Window for text widget. */
@@ -479,7 +479,7 @@ AlignPrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
 					 * information about how to reclaim
 					 * storage for return string. */
 {
-    Arg result = NULL;
+    Tcl_Obj * result = NULL;
     *freeProcPtr = (Tcl_FreeProc *) NULL;
     switch (((TkTextEmbImage *) widgRec)->align) {
 	case ALIGN_BASELINE:
