@@ -1139,15 +1139,19 @@ StartEnd(string, indexPtr)
 
 	segPtr = TkTextIndexToSeg(indexPtr, &offset);
 	while (1) {
+	    int chSize = 1;
+
 	    if (segPtr->typePtr == &tkTextCharType) {
-		c = segPtr->body.chars[offset];
-		if (!isalnum(UCHAR(c)) && (c != '_')) {
+		Tcl_UniChar ch;
+
+		chSize = Tcl_UtfToUniChar(segPtr->body.chars + offset, &ch);
+		if (!Tcl_UniCharIsWordChar(ch)) {
 		    break;
 		}
 		firstChar = 0;
 	    }
-	    offset += 1;
-	    indexPtr->byteIndex += sizeof(char);
+	    offset += chSize;
+	    indexPtr->byteIndex += chSize;
 	    if (offset >= segPtr->size) {
 		segPtr = TkTextIndexToSeg(indexPtr, &offset);
 	    }
@@ -1168,15 +1172,25 @@ StartEnd(string, indexPtr)
 
 	segPtr = TkTextIndexToSeg(indexPtr, &offset);
 	while (1) {
+	    int chSize = 1;
+
 	    if (segPtr->typePtr == &tkTextCharType) {
-		c = segPtr->body.chars[offset];
-		if (!isalnum(UCHAR(c)) && (c != '_')) {
+		Tcl_UniChar ch;
+
+		Tcl_UtfToUniChar(segPtr->body.chars + offset, &ch);
+		if (!Tcl_UniCharIsWordChar(ch)) {
 		    break;
 		}
+		if (offset > 0) {
+		    chSize = (segPtr->body.chars + offset
+			    - Tcl_UtfPrev(segPtr->body.chars + offset,
+			    segPtr->body.chars));
+		}
 		firstChar = 0;
+	    } else {
 	    }
-	    offset -= 1;
-	    indexPtr->byteIndex -= sizeof(char);
+	    offset -= chSize;
+	    indexPtr->byteIndex -= chSize;
 	    if (offset < 0) {
 		if (indexPtr->byteIndex < 0) {
 		    indexPtr->byteIndex = 0;
