@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclUnixFile.c,v 1.28 2002/09/03 02:01:42 hobbs Exp $
+ * RCS: @(#) $Id: tclUnixFile.c,v 1.32.2.2 2003/10/31 08:46:41 vincentdarley Exp $
  */
 
 #include "tclInt.h"
@@ -217,25 +217,26 @@ TclpMatchInDirectory(interp, resultPtr, pathPtr, pattern, types)
 
     if (pattern == NULL || (*pattern == '\0')) {
 	/* Match a file directly */
-	CONST char *native = (CONST char*) Tcl_FSGetNativePath(pathPtr);
+	native = (CONST char*) Tcl_FSGetNativePath(pathPtr);
 	if (NativeMatchType(native, types)) {
 	    Tcl_ListObjAppendElement(interp, resultPtr, pathPtr);
 	}
+	Tcl_DecrRefCount(fileNamePtr);
 	return TCL_OK;
     } else {
-	CONST char *fname, *dirName;
 	DIR *d;
-	Tcl_DString ds;
-	Tcl_StatBuf statBuf;
+	Tcl_DirEntry *entryPtr;
+	CONST char *dirName;
+	int dirLength;
 	int matchHidden;
 	int nativeDirLen;
-	int result = TCL_OK;
-	Tcl_DString dsOrig;
-	int baseLength;
+	Tcl_StatBuf statBuf;
+	Tcl_DString ds;      /* native encoding of dir */
+	Tcl_DString dsOrig;  /* utf-8 encoding of dir */
 
 	Tcl_DStringInit(&dsOrig);
-	Tcl_DStringAppend(&dsOrig, Tcl_GetString(fileNamePtr), -1);
-	baseLength = Tcl_DStringLength(&dsOrig);
+	dirName = Tcl_GetStringFromObj(fileNamePtr, &dirLength);
+	Tcl_DStringAppend(&dsOrig, dirName, dirLength);
 
 	/*
 	 * Make sure that the directory part of the name really is a
@@ -245,27 +246,17 @@ TclpMatchInDirectory(interp, resultPtr, pathPtr, pattern, types)
 	 * otherwise "glob foo.c" would return "./foo.c".
 	 */
 
-	if (baseLength == 0) {
+	if (dirLength == 0) {
 	    dirName = ".";
 	} else {
 	    dirName = Tcl_DStringValue(&dsOrig);
 	    /* Make sure we have a trailing directory delimiter */
-	    if (dirName[baseLength-1] != '/') {
+	    if (dirName[dirLength-1] != '/') {
 		dirName = Tcl_DStringAppend(&dsOrig, "/", 1);
-		baseLength++;
+		dirLength++;
 	    }
 	}
-
-	/*
-	 * Check to see if the pattern needs to compare with hidden files.
-	 */
-
-	if ((pattern[0] == '.')
-		|| ((pattern[0] == '\\') && (pattern[1] == '.'))) {
-	    matchHidden = 1;
-	} else {
-	    matchHidden = 0;
-	}
+	Tcl_DecrRefCount(fileNamePtr);
 
 	/*
 	 * Now open the directory for reading and iterate over the contents.
@@ -282,55 +273,36 @@ TclpMatchInDirectory(interp, resultPtr, pathPtr, pattern, types)
 
 	d = opendir(native);				/* INTL: Native. */
 	if (d == NULL) {
-	    char savedChar = '\0';
-	    Tcl_ResetResult(interp);
 	    Tcl_DStringFree(&ds);
-
-	    /*
-	     * Strip off a trailing '/' if necessary, before reporting the error.
-	     */
-
-	    if (baseLength > 0) {
-		savedChar = (Tcl_DStringValue(&dsOrig))[baseLength-1];
-		if (savedChar == '/') {
-		    (Tcl_DStringValue(&dsOrig))[baseLength-1] = '\0';
-		}
-	    }
+	    Tcl_ResetResult(interp);
 	    Tcl_AppendResult(interp, "couldn't read directory \"",
 		    Tcl_DStringValue(&dsOrig), "\": ",
 		    Tcl_PosixError(interp), (char *) NULL);
-	    if (baseLength > 0) {
-		(Tcl_DStringValue(&dsOrig))[baseLength-1] = savedChar;
-	    }
 	    Tcl_DStringFree(&dsOrig);
 	    return TCL_ERROR;
 	}
 
 	nativeDirLen = Tcl_DStringLength(&ds);
 
-	while (1) {
-	    Tcl_DString utfDs;
-	    CONST char *utf;
-	    Tcl_DirEntry *entryPtr;
+	/*
+	 * Check to see if -type or the pattern requests hidden files.
+	 */
+	matchHidden = ((types && (types->perm & TCL_GLOB_PERM_HIDDEN)) ||
+		((pattern[0] == '.')
+			|| ((pattern[0] == '\\') && (pattern[1] == '.'))));
 
-	    entryPtr = TclOSreaddir(d);			/* INTL: Native. */
-	    if (entryPtr == NULL) {
-		break;
-	    }
-	    if (types != NULL && (types->perm & TCL_GLOB_PERM_HIDDEN)) {
-		/*
-		 * We explicitly asked for hidden files, so turn around
-		 * and ignore any file which isn't hidden.
-		 */
-		if (*entryPtr->d_name != '.') {
-		    continue;
-		}
-	    } else if (!matchHidden && (*entryPtr->d_name == '.')) {
-		/*
-		 * Don't match names starting with "." unless the "." is
-		 * present in the pattern.
-		 */
-		continue;
+	while ((entryPtr = TclOSreaddir(d)) != NULL) { /* INTL: Native. */
+	    Tcl_DString utfDs;
+	    CONST char *utfname;
+
+	    /*
+	     * Skip this file if it doesn't agree with the hidden
+	     * parameters requested by the user (via -type or pattern).
+	     */
+	    if (*entryPtr->d_name == '.') {
+		if (!matchHidden) continue;
+	    } else {
+		if (matchHidden) continue;
 	    }
 
 	    /*
@@ -338,22 +310,20 @@ TclpMatchInDirectory(interp, resultPtr, pathPtr, pattern, types)
 	     * and pattern.  If so, add the file to the result.
 	     */
 
-	    utf = Tcl_ExternalToUtfDString(NULL, entryPtr->d_name, -1, &utfDs);
-	    if (Tcl_StringMatch(utf, pattern) != 0) {
+	    utfname = Tcl_ExternalToUtfDString(NULL, entryPtr->d_name,
+		    -1, &utfDs);
+	    if (Tcl_StringCaseMatch(utfname, pattern, 0)) {
 		int typeOk = 1;
 
-		Tcl_DStringSetLength(&dsOrig, baseLength);
-		Tcl_DStringAppend(&dsOrig, utf, -1);
-		fname = Tcl_DStringValue(&dsOrig);
 		if (types != NULL) {
-		    char *nativeEntry;
 		    Tcl_DStringSetLength(&ds, nativeDirLen);
-		    nativeEntry = Tcl_DStringAppend(&ds, entryPtr->d_name, -1);
-		    typeOk = NativeMatchType(nativeEntry, types);
+		    native = Tcl_DStringAppend(&ds, entryPtr->d_name, -1);
+		    typeOk = NativeMatchType(native, types);
 		}
 		if (typeOk) {
 		    Tcl_ListObjAppendElement(interp, resultPtr,
-			    Tcl_NewStringObj(fname, Tcl_DStringLength(&dsOrig)));
+			    TclNewFSPathObj(pathPtr, utfname,
+				    Tcl_DStringLength(&utfDs)));
 		}
 	    }
 	    Tcl_DStringFree(&utfDs);
@@ -362,7 +332,7 @@ TclpMatchInDirectory(interp, resultPtr, pathPtr, pattern, types)
 	closedir(d);
 	Tcl_DStringFree(&ds);
 	Tcl_DStringFree(&dsOrig);
-	return result;
+	return TCL_OK;
     }
 }
 static int
@@ -418,7 +388,19 @@ NativeMatchType(
 	    if (types->perm == 0) {
 		/* We haven't yet done a stat on the file */
 		if (TclOSstat(nativeEntry, &buf) != 0) {
-		    /* Posix error occurred */
+		    /*
+		     * Posix error occurred.  The only ok
+		     * case is if this is a link to a nonexistent
+		     * file, and the user did 'glob -l'. So
+		     * we check that here:
+		     */
+		    if (types->type & TCL_GLOB_TYPE_LINK) {
+			if (TclOSlstat(nativeEntry, &buf) == 0) {
+			    if (S_ISLNK(buf.st_mode)) {
+				return 1;
+			    }
+			}
+		    }
 		    return 0;
 		}
 	    }
@@ -765,10 +747,14 @@ TclpObjLink(pathPtr, toPtr, linkAction)
 	char link[MAXPATHLEN];
 	int length;
 	Tcl_DString ds;
+	Tcl_Obj *transPtr;
 
-	if (Tcl_FSGetTranslatedPath(NULL, pathPtr) == NULL) {
+	transPtr = Tcl_FSGetTranslatedPath(NULL, pathPtr);
+	if (transPtr == NULL) {
 	    return NULL;
 	}
+	Tcl_DecrRefCount(transPtr);
+
 	length = readlink(Tcl_FSGetNativePath(pathPtr), link, sizeof(link));
 	if (length < 0) {
 	    return NULL;
@@ -813,3 +799,26 @@ TclpFilesystemPathType(pathObjPtr)
     /* All native paths are of the same type */
     return NULL;
 }
+
+/*
+ *---------------------------------------------------------------------------
+ *
+ * TclpUtime --
+ *
+ *	Set the modification date for a file.
+ *
+ * Results:
+ *	0 on success, -1 on error.
+ *
+ * Side effects:
+ *	None.
+ *
+ *---------------------------------------------------------------------------
+ */
+int
+TclpUtime(pathPtr, tval)
+    Tcl_Obj *pathPtr;      /* File to modify */
+    struct utimbuf *tval;  /* New modification date structure */
+{
+    return utime(Tcl_FSGetNativePath(pathPtr),tval);
+}
