@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkUnixButton.c,v 1.10 2002/08/28 01:08:06 drh Exp $
+ * RCS: @(#) $Id: tkUnixButton.c,v 1.11.2.4 2003/10/10 20:20:47 hobbs Exp $
  */
 
 #include "tkButton.h"
@@ -86,12 +86,16 @@ TkpDisplayButton(clientData)
     int y, relief;
     Tk_Window tkwin = butPtr->tkwin;
     int width, height, fullWidth, fullHeight;
-    int imageXOffset, imageYOffset, textXOffset, textYOffset;
+    int textXOffset, textYOffset;
     int haveImage = 0, haveText = 0;
     int offset;			/* 1 means this is a button widget, so we
 				 * offset the text to make the button appear
 				 * to move up and down as the relief changes.
 				 */
+    int imageWidth, imageHeight;
+    int imageXOffset = 0, imageYOffset = 0; /* image information that will
+					     * be used to restrict disabled
+					     * pixmap as well */
 
     butPtr->flags &= ~REDRAW_PENDING;
     if ((butPtr->tkwin == NULL) || !Tk_IsMapped(tkwin)) {
@@ -176,11 +180,12 @@ TkpDisplayButton(clientData)
 	Tk_SizeOfBitmap(butPtr->display, butPtr->bitmap, &width, &height);
 	haveImage = 1;
     }
+    imageWidth  = width;
+    imageHeight = height;
+
     haveText = (butPtr->textWidth != 0 && butPtr->textHeight != 0);
 
     if (butPtr->compound != COMPOUND_NONE && haveImage && haveText) {
-	imageXOffset = 0;
-	imageYOffset = 0;
 	textXOffset = 0;
 	textYOffset = 0;
 	fullWidth = 0;
@@ -247,27 +252,27 @@ TkpDisplayButton(clientData)
 	    y += offset;
 	}
 
+	imageXOffset += x;
+	imageYOffset += y;
+
 	if (butPtr->image != NULL) {
 	    if ((butPtr->selectImage != NULL) && (butPtr->flags & SELECTED)) {
 		Tk_RedrawImage(butPtr->selectImage, 0, 0,
-			width, height, pixmap, x + imageXOffset,
-			y + imageYOffset);
+			width, height, pixmap, imageXOffset, imageYOffset);
 	    } else {
 		Tk_RedrawImage(butPtr->image, 0, 0, width,
-			height, pixmap, x + imageXOffset, y + imageYOffset);
+			height, pixmap, imageXOffset, imageYOffset);
 	    }
 	} else {
-	    XSetClipOrigin(butPtr->display, gc, x + imageXOffset,
-		    y + imageYOffset);
+	    XSetClipOrigin(butPtr->display, gc, imageXOffset, imageYOffset);
 	    XCopyPlane(butPtr->display, butPtr->bitmap, pixmap, gc,
-		    0, 0, (unsigned int) width,
-		    (unsigned int) height, x + imageXOffset,
-		    y + imageYOffset, 1);
+		    0, 0, (unsigned int) width, (unsigned int) height,
+		    imageXOffset, imageYOffset, 1);
 	    XSetClipOrigin(butPtr->display, gc, 0, 0);
 	}
 
-	Tk_DrawTextLayout(butPtr->display, pixmap, gc, butPtr->textLayout,
-		x + textXOffset, y + textYOffset, 0, -1);
+	Tk_DrawTextLayout(butPtr->display, pixmap, gc,
+		butPtr->textLayout, x + textXOffset, y + textYOffset, 0, -1);
 	Tk_UnderlineTextLayout(butPtr->display, pixmap, gc,
 		butPtr->textLayout, x + textXOffset, y + textYOffset,
 		butPtr->underline);
@@ -287,14 +292,16 @@ TkpDisplayButton(clientData)
 		x += offset;
 		y += offset;
 	    }
+	    imageXOffset += x;
+	    imageYOffset += y;
 	    if (butPtr->image != NULL) {
 		if ((butPtr->selectImage != NULL) &&
 			(butPtr->flags & SELECTED)) {
 		    Tk_RedrawImage(butPtr->selectImage, 0, 0, width,
-			    height, pixmap, x, y);
+			    height, pixmap, imageXOffset, imageYOffset);
 		} else {
 		    Tk_RedrawImage(butPtr->image, 0, 0, width, height, pixmap,
-			    x, y);
+			    imageXOffset, imageYOffset);
 		}
 	    } else {
 		XSetClipOrigin(butPtr->display, gc, x, y);
@@ -349,13 +356,22 @@ TkpDisplayButton(clientData)
 	    dim -= 2*butPtr->borderWidth;
 	    if (butPtr->flags & SELECTED) {
 		GC gc;
-		if (butPtr->state != STATE_DISABLED &&
-			butPtr->selectBorder != NULL) {
-		    gc = Tk_3DBorderGC(tkwin, butPtr->selectBorder,
-			    TK_3D_FLAT_GC);
+		if (butPtr->state != STATE_DISABLED) {
+		    if (butPtr->selectBorder != NULL) {
+			gc = Tk_3DBorderGC(tkwin, butPtr->selectBorder,
+				TK_3D_FLAT_GC);
+		    } else {
+			gc = Tk_3DBorderGC(tkwin, butPtr->normalBorder,
+				TK_3D_FLAT_GC);
+		    }
 		} else {
-		    gc = Tk_3DBorderGC(tkwin, butPtr->normalBorder,
-			    TK_3D_FLAT_GC);
+		    if (butPtr->disabledFg != NULL) {
+			gc = butPtr->disabledGC;
+		    } else {
+			gc = butPtr->normalTextGC;
+			XSetForeground(butPtr->display, butPtr->disabledGC,
+				Tk_3DBorderColor(butPtr->normalBorder)->pixel);
+		    }
 		}
 
 		XFillRectangle(butPtr->display, pixmap, gc, x, y,
@@ -381,11 +397,22 @@ TkpDisplayButton(clientData)
 	if (butPtr->flags & SELECTED) {
 	    GC gc;
 
-	    if (butPtr->state != STATE_DISABLED &&
-		    butPtr->selectBorder != NULL) {
-		gc = Tk_3DBorderGC(tkwin, butPtr->selectBorder,	TK_3D_FLAT_GC);
+	    if (butPtr->state != STATE_DISABLED) {
+		if (butPtr->selectBorder != NULL) {
+		    gc = Tk_3DBorderGC(tkwin, butPtr->selectBorder,
+			    TK_3D_FLAT_GC);
+		} else {
+		    gc = Tk_3DBorderGC(tkwin, butPtr->normalBorder,
+			    TK_3D_FLAT_GC);
+		}
 	    } else {
-		gc = Tk_3DBorderGC(tkwin, butPtr->normalBorder,	TK_3D_FLAT_GC);
+		if (butPtr->disabledFg != NULL) {
+		    gc = butPtr->disabledGC;
+		} else {
+		    gc = butPtr->normalTextGC;
+		    XSetForeground(butPtr->display, butPtr->disabledGC,
+			    Tk_3DBorderColor(butPtr->normalBorder)->pixel);
+		}
 	    }
 
 	    XFillPolygon(butPtr->display, pixmap, gc, points, 4, Convex,
@@ -403,23 +430,31 @@ TkpDisplayButton(clientData)
      * If the button is disabled with a stipple rather than a special
      * foreground color, generate the stippled effect.  If the widget
      * is selected and we use a different background color when selected,
-     * must temporarily modify the GC.
+     * must temporarily modify the GC so the stippling is the right color.
      */
 
     if ((butPtr->state == STATE_DISABLED)
 	    && ((butPtr->disabledFg == NULL) || (butPtr->image != NULL))) {
 	if ((butPtr->flags & SELECTED) && !butPtr->indicatorOn
 		&& (butPtr->selectBorder != NULL)) {
-	    XSetForeground(butPtr->display, butPtr->disabledGC,
+	    XSetForeground(butPtr->display, butPtr->stippleGC,
 		    Tk_3DBorderColor(butPtr->selectBorder)->pixel);
 	}
-	XFillRectangle(butPtr->display, pixmap, butPtr->disabledGC,
-		butPtr->inset, butPtr->inset,
-		(unsigned) (Tk_Width(tkwin) - 2*butPtr->inset),
-		(unsigned) (Tk_Height(tkwin) - 2*butPtr->inset));
+	/*
+	 * Stipple the whole button if no disabledFg was specified,
+	 * otherwise restrict stippling only to displayed image
+	 */
+	if (butPtr->disabledFg == NULL) {
+	    XFillRectangle(butPtr->display, pixmap, butPtr->stippleGC, 0, 0,
+		    (unsigned) Tk_Width(tkwin), (unsigned) Tk_Height(tkwin));
+	} else {
+	    XFillRectangle(butPtr->display, pixmap, butPtr->stippleGC,
+		    imageXOffset, imageYOffset,
+		    (unsigned) imageWidth, (unsigned) imageHeight);
+	}
 	if ((butPtr->flags & SELECTED) && !butPtr->indicatorOn
 		&& (butPtr->selectBorder != NULL)) {
-	    XSetForeground(butPtr->display, butPtr->disabledGC,
+	    XSetForeground(butPtr->display, butPtr->stippleGC,
 		    Tk_3DBorderColor(butPtr->normalBorder)->pixel);
 	}
     }
