@@ -39,8 +39,8 @@ typedef struct GroupItem  {
 
 static int		MembersParseProc _ANSI_ARGS_((ClientData clientData,
 			    Tcl_Interp *interp, Tk_Window tkwin,
-			    Arg value, char *recordPtr, int offset));
-static Arg		MembersPrintProc _ANSI_ARGS_((ClientData clientData,
+			    Tcl_Obj * value, char *recordPtr, int offset));
+static Tcl_Obj *		MembersPrintProc _ANSI_ARGS_((ClientData clientData,
 			    Tk_Window tkwin, char *recordPtr, int offset,
 			    Tcl_FreeProc **freeProcPtr));
 
@@ -1090,7 +1090,7 @@ MembersParseProc(clientData,interp,tkwin,value,recordPtr,offset)
 ClientData clientData;
 Tcl_Interp *interp;
 Tk_Window tkwin;
-Arg value;
+Tcl_Obj * value;
 char *recordPtr;
 int offset;
 {
@@ -1104,7 +1104,7 @@ int offset;
     return code;
 }
 
-static Arg
+static Tcl_Obj *
 MembersPrintProc(clientData,tkwin,recordPtr,offset,freeProcPtr)
 ClientData clientData;
 Tk_Window tkwin;
